/**
 * This file represents the pulsararchive database in mysql++
 **/


#include <mysql++.h>
#include <Pulsar/Archive.h>



using mysqlpp::Exception;
using mysqlpp::Connection;
using std::string;
using std::vector;
using Pulsar::Archive;





struct ObservationEntry
{
  string filename;
  string data_loc;
  string prefix;
  vector<string> files;
};

struct ObservationDetails
{
  string src_name;
  string project_id;
  string raj;
  string decj;
  string data_type;
  double obsfreq;
  double bw;
  double scanlen;
  string date;
  string ut;
  double MJD;
  double rajd;
  double decjd;
  double gl;
  double gb;
  float bmaj;
  float bmin;
  float bpa;
  double dm;
  double period;
  int nchan;
  int npol;
  int nbin;
  int nsub;
  float tsamp;
  int nbits;
  int nbeam;
  string cnfg;
//   string inst;
  string rcvr;
  string hdrver;
  string survey;
  string telescope;
  string site;
  string observer;
  long file_size_bytes;
};




/**
 * Exceptions the database might throw
 **/

class MYSQLPP_EXPORT FailedConnection : public Exception
{
  public:
    explicit FailedConnection(const char* w = "Unable to connect to database") : Exception(w) { }
};

class MYSQLPP_EXPORT GenericException : public Exception
{
  public:
    explicit GenericException( const char *w = "Unknown Expception" ) : Exception(w) { }
};





class psrdb
{
  public:
    psrdb();
    
    void Connect( const char *db_name, const char *db_hostname, const char *db_username, const char *db_password );
    
    void MarkObservations();
    
    void GetArchiveDetails( ObservationDetails &details, Reference::To<Archive> arch );
    
    void PurgeMissingObservations();
    void PurgeMissingCals();
    
    void GetObsDetails( const ObservationEntry &entry, ObservationDetails &details );
    
    void CheckObservation( const ObservationEntry &entry ) { CheckObsCals( entry, "observations" ); }
    void CheckCalibration( const ObservationEntry &entry ) { CheckObsCals( entry, "cals" ); }
  private:
    void CheckObsCals( const ObservationEntry &entry, string table_name );
    
    void CalculateGalactic( double rajd, double decjd, double &gl, double &gb );
    
    Connection db;
};



